"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.bindController = void 0;
const common_1 = require("@nestjs/common");
const bindings_1 = require("../../config/bindings");
const constants_1 = require("../../constants");
function bindController(controller, entity, connectionName, config) {
    for (const methodName in bindings_1.Bindings) {
        const { name, path, parameters, method, implementation } = bindings_1.Bindings[methodName];
        const decoratorOptions = Reflect.getMetadata(constants_1.JSON_API_DECORATOR_OPTIONS, controller);
        if (decoratorOptions) {
            const { allowMethod = Object.keys(bindings_1.Bindings) } = decoratorOptions;
            if (!allowMethod.includes(name))
                continue;
        }
        let implementationResultFunction = implementation;
        // Dirty hack, @nestjs/swagger will crash, because descriptor is empty for inherite class
        if (controller.prototype[name]) {
            implementationResultFunction = controller.prototype[name];
        }
        controller.prototype[name] = function (...args) {
            // @ts-ignore
            return implementationResultFunction.call(this, ...args);
        };
        const descriptor = Object.getOwnPropertyDescriptor(controller.prototype, name);
        switch (method) {
            case common_1.RequestMethod.GET: {
                (0, common_1.Get)(path)(controller.prototype, name, descriptor);
                break;
            }
            case common_1.RequestMethod.DELETE: {
                (0, common_1.HttpCode)(204)(controller.prototype, name, descriptor);
                (0, common_1.Delete)(path)(controller.prototype, name, descriptor);
                break;
            }
            case common_1.RequestMethod.POST: {
                (0, common_1.Post)(path)(controller.prototype, name, descriptor);
                break;
            }
            case common_1.RequestMethod.PATCH: {
                (0, common_1.Patch)(path)(controller.prototype, name, descriptor);
                break;
            }
            default: {
                throw new Error(`Method '${method}' unsupported`);
            }
        }
        for (const key in parameters) {
            const parameter = parameters[key];
            const { property, decorator, mixins } = parameter;
            const resultMixin = mixins.map((mixin) => mixin(entity, connectionName, config));
            decorator(property, ...resultMixin)(controller.prototype, name, parseInt(key, 10));
        }
    }
}
exports.bindController = bindController;
//# sourceMappingURL=index.js.map